#
# Patch unrarsrc-3.3.6 to compile with DJGPP.
# After applying the patch issue the command
# make -f makefile.dj
# If you don't have UPX remove the upx command from makefile.dj and uncomment
# the strip command.
#
# NOTE: The patch against find.cpp fixes a bug where the file access mode of
# the archive to be extracted was set to read-only (DOS effect of chmod 0).
# attr=chmod(name,0) seemed to be intended to be written as attr=_chmod(name,0)
# (note the leading underscore), a DJGPP extension to simply get the file mode
# instead of using a quite expensive stat() call. But since we already did a
# stat() before we don't need to redo anything again..
#
# Sat Jan 18 13:59:44 CET 2003
# Eike Rathke <er@erack.de>
#
# 18.03.04 14:49:50 Aliaksandr Lakhanko <alex@eunet.lt>:
# - build with djgpp v2.04-beta1 gcc v3.3.3
#   correctly set unpacked file time in win2k dos box
# - added stub cwsdstub.exe
#   work without cwsdpmi.exe in pure dos
#
diff -u -N unrar/filefn.cpp unrar.dj/filefn.cpp
--- unrar/filefn.cpp	Fri Jan 30 16:26:20 2004
+++ unrar.dj/filefn.cpp	Wed Mar 17 20:21:08 2004
@@ -189,6 +189,7 @@
   return(1457664);
 #elif defined(_EMX)
   int Drive=(!isalpha(Name[0]) || Name[1]!=':') ? 0:toupper(Name[0])-'A'+1;
+#ifndef _DJGPP
   if (_osmode == OS2_MODE)
   {
     FSALLOCATE fsa;
@@ -199,12 +200,17 @@
     return(FreeSize);
   }
   else
+#endif
   {
     union REGS regs,outregs;
     memset(&regs,0,sizeof(regs));
     regs.h.ah=0x36;
     regs.h.dl=Drive;
+#ifdef _DJGPP
+    int86 (0x21,&regs,&outregs);
+#else
     _int86 (0x21,&regs,&outregs);
+#endif
     if (outregs.x.ax==0xffff)
       return(1457664);
     Int64 FreeSize=outregs.x.ax*outregs.x.cx;
diff -u -N unrar/find.cpp unrar.dj/find.cpp
--- unrar/find.cpp	Fri Jan 30 16:26:20 2004
+++ unrar.dj/find.cpp	Thu Mar 18 14:43:32 2004
@@ -149,9 +149,7 @@
       fd->Error=(errno!=ENOENT);
       return(false);
     }
-#ifdef _DJGPP
-  fd->FileAttr=chmod(FindMask,0);
-#elif defined(_EMX)
+#if defined(_EMX) && !defined(_DJGPP)
   fd->FileAttr=st.st_attr;
 #else
   fd->FileAttr=st.st_mode;
diff -u -N unrar/makefile.dj unrar.dj/makefile.dj
--- unrar/makefile.dj	Thu Jan 01 02:00:00 1970
+++ unrar.dj/makefile.dj	Thu Mar 18 14:35:38 2004
@@ -0,0 +1,53 @@
+#
+# Makefile for DOS DJGPP - unrar
+#
+# If you don't have UPX remove the upx call and uncomment the strip call.
+#
+# Note: you have to 'make clean' before you can build
+#	the sfx module
+#
+
+# DOS using DJGPP
+CXX=gpp
+CXXFLAGS=-O2 -Wno-deprecated
+DEFINES=
+#-D_FILE_OFFSET_BITS=64 -D_LARGEFILE_SOURCE
+
+##########################
+
+COMPILE=$(CXX) $(CXXFLAGS) $(DEFINES)
+LINK=$(CXX)
+
+UNRAR_OBJ=filestr.o recvol.o rs.o scantree.o
+
+OBJECTS=rar.o strlist.o strfn.o pathfn.o int64.o savepos.o global.o file.o filefn.o filcreat.o \
+	archive.o arcread.o unicode.o system.o isnt.o crypt.o crc.o rawread.o encname.o \
+	resource.o match.o timefn.o rdwrfn.o consio.o options.o ulinks.o errhnd.o rarvm.o \
+	rijndael.o getbits.o sha1.o extinfo.o extract.o volume.o list.o find.o unpack.o cmddata.o
+
+.cpp.o:
+	$(COMPILE) -D$(WHAT) -c $<
+
+all:	unrar.exe
+
+clean:
+	-rm *.o unrar.exe default.sfx
+
+unrar.exe:	WHAT=UNRAR
+unrar.exe:	$(OBJECTS) $(UNRAR_OBJ)
+#	@rm -f makeunrar
+	$(LINK) -o unrar.exe $(LDFLAGS) $(OBJECTS) $(UNRAR_OBJ) $(LIBS)
+	strip unrar.exe
+	upx -9 unrar.exe
+	exe2coff unrar.exe
+	cp $(DJDIR)/bin/cwsdstub.exe stub
+	copy /b stub+unrar unrar.exe
+	rm stub
+	rm unrar
+
+sfx:	WHAT=SFX_MODULE
+sfx:	$(OBJECTS)
+#	@rm -f default.sfx
+	$(LINK) -o default.sfx $(LDFLAGS) $(OBJECTS) -DSFX_MODULE
+	strip default.sfx
+
diff -u -N unrar/pathfn.cpp unrar.dj/pathfn.cpp
--- unrar/pathfn.cpp	Fri Jan 30 16:26:20 2004
+++ unrar.dj/pathfn.cpp	Thu Mar 18 14:42:46 2004
@@ -294,6 +294,7 @@
     strcpy(RARFileName,Path);
   if (Number!=0)
     return(false);
+#ifndef _DJGPP
   if (_osmode==OS2_MODE)
   {
     PTIB ptib;
@@ -302,6 +303,7 @@
     DosQueryModuleName(ppib->pib_hmte,NM,Path);
   }
   else
+#endif
     strcpy(Path,RARFileName);
   RemoveNameFromPath(Path);
   return(true);
